#include "StdAfx.h"
#include "Application.h"

CApplication * CApplication::instance = NULL;

CApplication::CApplication(void)
{
}

CApplication::~CApplication(void)
{
}

void CApplication::Start(void) {
	InitWindow();
	InitGL();

	CScene * scene = new CTestScene();
	scene->Init();
	scene->Activate();

	GameLoop();

	Cleanup();
}

void CApplication::InitWindow(void) {
	glfwInit();
	glfwOpenWindowHint(GLFW_FSAA_SAMPLES, 8);
	glfwOpenWindowHint(GLFW_OPENGL_VERSION_MAJOR, OPENGL_MAJOR);
	glfwOpenWindowHint(GLFW_OPENGL_VERSION_MINOR, OPENGL_MINOR);
	glfwOpenWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);
	glfwOpenWindow(640, 480, 8, 8, 8, 8, 24, 0, GLFW_WINDOW);
	glfwSetWindowPos(300, 200);
	glfwSetWindowTitle("PGK Lab 2");
	glfwSetTime(0.0);
	glfwSetWindowSizeCallback(OnWindowSize);
	glfwSetMousePosCallback(CInput::OnMouseMove);
	//glfwDisable(GLFW_MOUSE_CURSOR);
	glfwSetMousePos(320, 240);
	CInput::GetInstance()->MouseX = 320;
	CInput::GetInstance()->MouseY = 240;
}

void CApplication::InitGL(void) {
	int w, h;
	glfwGetWindowSize(&w, &h);
	glViewport(0, 0, w, h);
	glClearColor(0, 0, 0, 0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glewExperimental = GL_TRUE;
	glewInit();
}

void CApplication::ProcessEvents(void) {
	CInput::GetInstance()->HandleInput();
}

void CApplication::Cleanup(void) {
	glfwTerminate();
}

double CApplication::GetTime(void) {
	return glfwGetTime();
}


void CApplication::Redraw() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	CScene::GetCurrent()->Redraw();

	glfwSwapBuffers();
}


void CApplication::Update() {
	ProcessEvents();
	CScene::GetCurrent()->Update();
}


void CApplication::GameLoop() {
	
	double currentTime = GetTime();
	double nextUpdateTime = GetTime();

	int MaxFrameskip = 10;

	while (!CInput::GetInstance()->UserWantsToExit()) {
		currentTime = GetTime();

		int substeps = 0;
		while ((currentTime >= nextUpdateTime) && (substeps < MaxFrameskip)) {
			Update();
			++substeps;
			nextUpdateTime += UPDATE_INTERVAL;
		}

		Redraw();
	}

}

void GLFWCALL CApplication::OnWindowSize(int w, int h) {
	GetInstance()->WindowWidth = w;
	GetInstance()->WindowHeight = h;
	glViewport(0, 0, w, h);
	if (CScene::GetCurrent() != NULL) {
		CScene::GetCurrent()->Camera.SetViewport(w, h);
	}
}

