#include "StdAfx.h"
#include "ParticleSpawner.h"

// Konstruktor
CParticleSpawner::CParticleSpawner(void) : CRenderableNode()
{
	Primitive = GL_POINTS; // Prymityw ktry zostanie uyty przez CRenderableNode podczas tworzenia dania renderowania
	MaxParticles = 1000; // Maksymalna liczba czstek widocznych w jednej chwili - od tej liczby zaley ilo zarezerwowanej pamici dla VBO
	ParticleSize = glm::vec2(1.5f, 1.5f); // Rozmiar czstki tworzonej przez Geometry Shader w miejscu kadego punktu
}

// Destruktor
CParticleSpawner::~CParticleSpawner(void)
{
	Particles.clear();
}

// Program cieniujcy uywany do renderowania czstek
CShadingProgram *CParticleSpawner::ParticleShadingProgram = NULL;

// Inicjalizacja
void CParticleSpawner::Init(void) {
	CRenderableNode::Init();

	// Utworzenie programu cieniujcego skadajcego si z VS, GS i FS:
	if (ParticleShadingProgram == NULL) {
		ParticleShadingProgram = new CShadingProgram();
		ParticleShadingProgram->AttachShader(GL_VERTEX_SHADER, "Shaders/Particle.vert");
		ParticleShadingProgram->AttachShader(GL_GEOMETRY_SHADER, "Shaders/Particle.geom");
		ParticleShadingProgram->AttachShader(GL_FRAGMENT_SHADER, "Shaders/Particle.frag");
	}
	ShadingProgram = ParticleShadingProgram;
}

// Aktualizacja stanu systemu czsteczkowego - najwaniejsze metoda zawierajca ca logik dziaania
void CParticleSpawner::Update(void) {
	CRenderableNode::Update();

	// Tutaj naley umieci logik systemu czsteczkowego. Caa ponisza zawarto tej metody
	// tworzy jedn "czsteczk" w miejscu kuli ktra yje tak dugo, jak uytkownik wciska spacj.
	
	Particles.clear();

	if (CRegistry::GetInstance()->EmitSmoke) {
		SParticle *p = new SParticle();
		p->Position = Position + glm::vec3(0, 1, 0);
		Particles.push_back(p);
	}
}

// Utworzenie i inicjalizacja VBO ktre bdzie zawierao aktualny stan systemu czsteczkowego
void CParticleSpawner::PrepareGeometryBuffer(void) {
	CRenderableNode::PrepareGeometryBuffer();
	glBufferData(GL_ARRAY_BUFFER, VBO_STRIDE * MaxParticles * sizeof(GLfloat), NULL, GL_STREAM_DRAW);
	UpdateGeometryBuffer();
}

// Aktualizacja zawartoci VBO na podstawie tablicy czstek
void CParticleSpawner::UpdateGeometryBuffer(void) {

	NumVertices = Particles.size(); // Aktualna liczba czstek w tablicy

	GLfloat * buf = new GLfloat[VBO_STRIDE * NumVertices]; // Przygotowanie bufora na now zawarto VBO - tylko dla aktualnej liczby czstek
	
	// Wypenienie bufora now zawartoci VBO:
	// Iteracja po wszystkich czstkach w tablicy
	int i = 0;
	for (vector<SParticle *>::iterator it = Particles.begin(); it != Particles.end(); ++it, ++i) {
		buf[i * VBO_STRIDE + 0] = (*it)->Position.x;
		buf[i * VBO_STRIDE + 1] = (*it)->Position.y;
		buf[i * VBO_STRIDE + 2] = (*it)->Position.z;
	}
	
	// Wysanie nowej zawartoci VBO do karty graficznej
	glBufferSubData(GL_ARRAY_BUFFER, 0, VBO_STRIDE * NumVertices * sizeof(GLfloat), buf);

	// Usunicie zawartoci bufora z pamici gwnej
	delete[] buf;
}

// Aktualizacja macierzy Modelu - przysonicie metody z CRenderableNode ktra uwzgldnia translacj i rotacj caego emitera czstek.
// Nie chcemy, by miay one wpyw na pozycje czstek po ich wyemitowaniu. Czstki s niezalene, wic sama ich pozycja musi uwzgldnia
// transformacj emitera w momencie emisji.
void CParticleSpawner::UpdateModelMatrix(void) {
	ModelMatrix = glm::mat4(1.0f);
}

// Ustalenie wartoci niestandardowych zmiennych dla shaderw
void CParticleSpawner::BindUniforms(void) {
	CRenderableNode::BindUniforms();
	glUniform2fv(CShadingProgram::GetCurrent()->GetUniformLocation(GLSL_UNIFORM_NAME_PARTICLE_SIZE), 1, glm::value_ptr(ParticleSize)); // Rozmiar czstki dla GS
}

// Przygotowanie wskanikw na atrybuty wierzchokw umieszczone w VBO
void CParticleSpawner::PrepareGeometryPointers(void) {
	CRenderableNode::PrepareGeometryPointers();
	
	// Pozycja wierzchoka (czstki):
	glVertexAttribPointer(_GLSL_VERTEX_POSITION_POS, 3, GL_FLOAT, GL_FALSE, VBO_STRIDE * sizeof(GLfloat), 0);
	glEnableVertexAttribArray(_GLSL_VERTEX_POSITION_POS);
}

// Renderowanie czstek
void CParticleSpawner::Render(void) {
	UpdateGeometryBuffer();
	glDepthMask(GL_FALSE); // Nie chcemy, by czstki ktre w zaoeniu bd na wp przezroczyste, modyfikoway zawarto bufora gbokoci
	CRenderableNode::Render();
	glDepthMask(GL_TRUE);
}
