#include "StdAfx.h"
#include "RenderableNode.h"


CRenderableNode::CRenderableNode(void) : CNode() {
	VBO_STRIDE = 32;
	Primitive = GL_TRIANGLES;
	ShadingProgram = NULL;
}


CRenderableNode::~CRenderableNode(void) {
}

void CRenderableNode::Draw(void) {
	if (ShadingProgram == NULL) {
		ShadingProgram = Scene->SP_General;
	}
	ShadingProgram->Activate();

	UpdateModelMatrix();
	BindUniforms();
	Scene->BindLights();
	glBindVertexArray(vaoId);
	Render();
}

void CRenderableNode::UpdateModelMatrix(void) {
	ModelMatrix = glm::rotate(Rotation.x, glm::vec3(1, 0, 0)) * glm::rotate(Rotation.y, glm::vec3(0, 1, 0)) * glm::rotate(Rotation.z, glm::vec3(0, 0, 1)) * glm::translate(Position);
}

void CRenderableNode::BindUniforms(void) {
	glm::mat4 ModelViewMatrix = Scene->Camera.ViewMatrix * ModelMatrix;
	glm::mat4 ModelViewProjectionMatrix = Scene->Camera.ProjectionMatrix * ModelViewMatrix;
	glm::mat3 NormalMatrix(ModelViewMatrix);
	glUniformMatrix4fv(ShadingProgram->GetUniformLocation(GLSL_UNIFORM_NAME_MODELVIEWPROJECTION_MATRIX), 1, GL_FALSE, glm::value_ptr(ModelViewProjectionMatrix));
	glUniformMatrix4fv(ShadingProgram->GetUniformLocation(GLSL_UNIFORM_NAME_MODELVIEW_MATRIX), 1, GL_FALSE, glm::value_ptr(ModelViewMatrix));
	glUniformMatrix3fv(ShadingProgram->GetUniformLocation(GLSL_UNIFORM_NAME_NORMAL_MATRIX), 1, GL_FALSE, glm::value_ptr(NormalMatrix));
	glUniformMatrix4fv(ShadingProgram->GetUniformLocation(GLSL_UNIFORM_NAME_PROJECTION_MATRIX), 1, GL_FALSE, glm::value_ptr(Scene->Camera.ProjectionMatrix));
}

void CRenderableNode::Init(void) {
	CNode::Init();
	PrepareGeometryBuffer();
	PrepareGeometryPointers();
}

void CRenderableNode::PrepareGeometryBuffer(void) {
	glGenBuffers(1, &vboId);
	glBindBuffer(GL_ARRAY_BUFFER, vboId);
}

void CRenderableNode::PrepareGeometryPointers(void) {
	glGenVertexArrays(1, &vaoId);
	glBindVertexArray(vaoId);
	glBindBuffer(GL_ARRAY_BUFFER, vboId);
}

void CRenderableNode::Render(void) {
	glDrawArrays(Primitive, 0, NumVertices);
}
