#include "StdAfx.h"
#include "TestScene.h"


CTestScene::CTestScene(void) : CScene()
{
}


CTestScene::~CTestScene(void)
{
}

void CTestScene::Init(void) {
	CScene::Init();

	Camera.SetPosition(glm::vec3(0.0f, 0.0f, 10.0f));

	// Add sphere
	{
		CNode * node = new CSphere(5);
		node->Position = glm::vec3(0.0f, 0.0f, 0.0f);
		AddNode(node);
	}
	
	// Add circulating light
	{
		CLight * l = new CLight();
		l->Scene = this;
		l->Position = glm::vec3(0.0f, 5.0f, 0.0f);
		l->Ambient = glm::vec3(0.0f, 0.0f, 0.0f);
		l->Diffuse = glm::vec3(1.0f, 0.6f, 0.1f);
		l->Specular = glm::vec3(1.0f, 1.0f, 1.0f);
		Lights.push_back(l);
	}

	// Add bottom light
	{
		CLight * l = new CLight();
		l->Scene = this;
		l->Position = glm::vec3(0.0f, -5.0f, 0.0f);
		l->Ambient = glm::vec3(0.0f, 0.0f, 0.0f);
		l->Diffuse = glm::vec3(1.0f, 0.0f, 0.0f);
		l->Specular = glm::vec3(0.5f, 0.2f, 0.2f);
		Lights.push_back(l);
	}

	// Add smoke spawner
	{
		CParticleSpawner * p = new CParticleSpawner();
		p->Position = glm::vec3(0.0f, 0.0f, -1.0f);
		AddNode(p);
	}

}

void CTestScene::Redraw(void) {
	SP_General->Activate();

	// Rotate the light
	{
		Lights[0]->Position.x = 5.0f * glm::cos(Time);
		Lights[0]->Position.z = 5.0f * glm::sin(Time);
	}

	CScene::Redraw();
}

void CTestScene::Update(void) {
	CScene::Update();

	// Move sphere with mouse cursor
	{
		int mx, my, wx, wy;
		mx = CInput::GetInstance()->MouseX;
		my = CInput::GetInstance()->MouseY;
		wx = CApplication::GetInstance()->WindowWidth;
		wy = CApplication::GetInstance()->WindowHeight;
		for (std::vector<CNode *>::iterator it = Nodes.begin(); it != Nodes.end(); ++it) {
			(*it)->Position.x = ((2.0f * mx / wx) - 1.0f) * 3.0f;
			(*it)->Position.y = -((2.0f * my / wy) - 1.0f) * 3.0f;
		}
	}
}

